import os
from . import files
from . import gametimer
from . import video
from . import models
from . import roc_main

class anim(object):
	def __init__(self, frameset):
		self.frame = 0
		self.frameset = frameset
		self.frame_timer = self.frameset.frame_delays[0]
		self.dirty = True
		self.repeats = None
		self.done = False

	def update(self):
		self.frame_timer -= roc_main.get_timer().elapsed
		while self.frame_timer <= 0:
			self.dirty = True
			self.frame += 1
			if len(self.frameset.frames) > 2:
				print("Updating frame on %s to %s" % (self.frameset.frames[0].id, self.frame))
			if self.frame >= len(self.frameset.frames):
				if self.repeats == None:
					"repeats == None means repeat forever"
					pass
				elif self.repeats == 0:
					"no more repeats, stop playing animation and leave it on the last frame"
					self.frame = len(self.frameset.frames) - 1
					self.done = True
					break
				else:
					self.repeats -= 1
				self.frame = 0
			self.frame_timer += self.frameset.frame_delays[self.frame]

		return self.dirty

	def draw(self):
		self.frameset.draw(self.frame)
		self.dirty = False

	def size(self):
		return self.frameset.size()



class anim_frameset(object):
	def __init__(self, first_img, delay):
		self.frames = []
		self.sprite = None
		self.load_frames(first_img)
		self.cached_surface = None

		self.frame_delays = []
		self.set_delay(delay)

	def load_frames(self, iff):
		ifn, ife = os.path.splitext(iff)
		fnp, fnn = os.path.split(ifn)
		if ifn and ifn[-1] == '1':
			self.frames = []

			ifn = ifn[:-1]
			fnpath, fnbase = os.path.split(ifn)
			padlen = 0
			while ifn[-1] == '0':
				ifn = ifn[:-1]
				padlen += 1

			i = 1
			def padnum(n, l):
				return (("0" * max(0,l-len(str(n))+1)) + str(n))
			while True:
				fnnum = fnbase + padnum(i, padlen)
				iff = ifn + padnum(i, padlen) + ife
				if files.mgr.exists(iff):
					#LOAD
					mdl = models.mgr.create_sprite(fnnum, iff)
					self.frames.append(mdl)
					if self.sprite == None:
						self.sprite = models.Model()
						self.sprite.id = fnbase
						self.sprite.add_sprite_layer(iff)
					#self.frames += [files.mgr.png(iff)]
				else:
					break
				i += 1
		else:
			#LOAD
			mdl = models.mgr.create_sprite(fnn, iff)
			self.frames.append(mdl)
			self.sprite = mdl
			#self.frames = [files.mgr.png(ifn + ife)]
		
		if not self.frames:
			raise OSError('File "%s" does not exist' % (ifn + ife))


	def set_delay(self, delay):
		try:
			iter(delay)
		except TypeError:
			delay = [int(delay)] * len(self.frames)
		self.frame_delays = [int(x) for x in delay]


	def size(self):
		sc = self.frames[0].layers[0].scale
		return int(sc[0]), int(sc[1])
	

	def draw(self, frame):
		self.sprite.layers[0].material = self.frames[frame].layers[0].material
		self.sprite.render()
"""
	def draw(self, scr, pos, frame):
		if scr != self.cached_surface:
			for f in self.frames:
				f.convert_alpha(scr)
			self.cached_surface = scr

		img = self.frames[frame]
		scr.blit(img, pos)
"""



