import warnings
import functools


def deprecated(func):
	"""This is a decorator which can be used to mark functions
	as deprecated. It will result in a warning being emitted
	when the function is used."""
	
	@functools.wraps(func)
	def new_func(*args, **kwargs):
		warnings.warn(
			"Call to deprecated function %(funcname)s." % {
				'funcname': func.__name__,
			},
			category=DeprecationWarning,
			stacklevel=2
		)
		return func(*args, **kwargs)
	return new_func
